import gleam/dynamic/decode.{type Decoder}
import gleam/json.{type Json}

pub type EngineSymbol {
  EngineImpulseDriveI
  EngineIonDriveI
  EngineIonDriveII
  EngineHyperDriveI
}

pub fn parse(value: String) -> Result(EngineSymbol, Nil) {
  case value {
    "ENGINE_IMPULSE_DRIVE_I" -> Ok(EngineImpulseDriveI)
    "ENGINE_ION_DRIVE_I" -> Ok(EngineIonDriveI)
    "ENGINE_ION_DRIVE_II" -> Ok(EngineIonDriveII)
    "ENGINE_HYPER_DRIVE_I" -> Ok(EngineHyperDriveI)
    _ -> Error(Nil)
  }
}

pub fn decoder() -> Decoder(EngineSymbol) {
  use value <- decode.then(decode.string)
  case parse(value) {
    Ok(engine_symbol) -> decode.success(engine_symbol)
    Error(Nil) -> decode.failure(EngineImpulseDriveI, "EngineSymbol")
  }
}

pub fn to_string(engine_symbol: EngineSymbol) -> String {
  case engine_symbol {
    EngineImpulseDriveI -> "ENGINE_IMPULSE_DRIVE_I"
    EngineIonDriveI -> "ENGINE_ION_DRIVE_I"
    EngineIonDriveII -> "ENGINE_ION_DRIVE_II"
    EngineHyperDriveI -> "ENGINE_HYPER_DRIVE_I"
  }
}

pub fn encode(engine_symbol: EngineSymbol) -> Json {
  json.string(to_string(engine_symbol))
}
