import gleam/dynamic/decode.{type Decoder}
import gleam/json.{type Json}

pub type ShipComponent {
  Frame
  Reactor
  Engine
}

pub fn parse(value: String) -> Result(ShipComponent, Nil) {
  case value {
    "FRAME" -> Ok(Frame)
    "REACTOR" -> Ok(Reactor)
    "ENGINE" -> Ok(Engine)
    _ -> Error(Nil)
  }
}

pub fn decoder() -> Decoder(ShipComponent) {
  use value <- decode.then(decode.string)
  case parse(value) {
    Ok(ship_component) -> decode.success(ship_component)
    Error(Nil) -> decode.failure(Frame, "ShipComponent")
  }
}

pub fn to_string(ship_component: ShipComponent) -> String {
  case ship_component {
    Frame -> "FRAME"
    Reactor -> "REACTOR"
    Engine -> "ENGINE"
  }
}

pub fn encode(ship_component: ShipComponent) -> Json {
  json.string(to_string(ship_component))
}
