import gleam/dynamic/decode.{type Decoder}
import gleam/json.{type Json}

pub type ShipRole {
  Fabricator
  Harvester
  Hauler
  Interceptor
  Excavator
  Transport
  Repair
  Surveyor
  Command
  Carrier
  Patrol
  Satellite
  Explorer
  Refinery
}

pub fn parse(value: String) -> Result(ShipRole, Nil) {
  case value {
    "FABRICATOR" -> Ok(Fabricator)
    "HARVESTER" -> Ok(Harvester)
    "HAULER" -> Ok(Hauler)
    "INTERCEPTOR" -> Ok(Interceptor)
    "EXCAVATOR" -> Ok(Excavator)
    "TRANSPORT" -> Ok(Transport)
    "REPAIR" -> Ok(Repair)
    "SURVEYOR" -> Ok(Surveyor)
    "COMMAND" -> Ok(Command)
    "CARRIER" -> Ok(Carrier)
    "PATROL" -> Ok(Patrol)
    "SATELLITE" -> Ok(Satellite)
    "EXPLORER" -> Ok(Explorer)
    "REFINERY" -> Ok(Refinery)
    _ -> Error(Nil)
  }
}

pub fn decoder() -> Decoder(ShipRole) {
  use value <- decode.then(decode.string)
  case parse(value) {
    Ok(ship_role) -> decode.success(ship_role)
    Error(Nil) -> decode.failure(Fabricator, "ShipRole")
  }
}

pub fn to_string(ship_role: ShipRole) -> String {
  case ship_role {
    Fabricator -> "FABRICATOR"
    Harvester -> "HARVESTER"
    Hauler -> "HAULER"
    Interceptor -> "INTERCEPTOR"
    Excavator -> "EXCAVATOR"
    Transport -> "TRANSPORT"
    Repair -> "REPAIR"
    Surveyor -> "SURVEYOR"
    Command -> "COMMAND"
    Carrier -> "CARRIER"
    Patrol -> "PATROL"
    Satellite -> "SATELLITE"
    Explorer -> "EXPLORER"
    Refinery -> "REFINERY"
  }
}

pub fn encode(ship_role: ShipRole) -> Json {
  json.string(to_string(ship_role))
}
